use strict;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw (
    ExistsObjectByPath
    LoadObjectByPath
    LoadRootObject
);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );
use DE_EPAGES::UPS::API::DataCollectors;
use DE_EPAGES::UPS::API::XML::Builder::AddressBuilder;
use Readonly;
use Test::More;
use Test::Exception;
use Data::Dumper;

Readonly my $ROOTPATH => '/TestStore';
## no critic (ProhibitPrivateSubroutineUse ProhibitLongLines)

sub Main {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {

            # startUp
            ok( Init(), 'initialization' );

            # tests
            eval {
                my $ShippingLabelUPS = LoadObjectByPath($ROOTPATH.'/TestShop/Customers/1001/Orders/1002/ShippingLabels/1001');## no critic (ProhibitPathConcatenation)
                isa_ok($ShippingLabelUPS, 'DE_EPAGES::UPS::API::Object::ShippingLabelUPS');
                my $DataCollectors = DE_EPAGES::UPS::API::DataCollectors->new($ShippingLabelUPS);
                isa_ok($ShippingLabelUPS->get('Order'), 'DE_EPAGES::Order::API::Object::CustomerOrder');
                isa_ok($DataCollectors, 'DE_EPAGES::UPS::API::DataCollectors');
                my $hShippingAddress = $DataCollectors->getAddress('Address');
                my $AddressBuilder = DE_EPAGES::UPS::API::XML::Builder::AddressBuilder->new($hShippingAddress);
                isa_ok($AddressBuilder, 'DE_EPAGES::UPS::API::XML::Builder::AddressBuilder');
                testPhone($AddressBuilder);
                testAttentionName($AddressBuilder);
                is ($AddressBuilder->_addressLine()->{'AddressLine'},'Blumenweg 2','call _addressLine with valid Value');
                is ($AddressBuilder->_name()->{'Name'},'Max Mustermann','call _name with valid Value');
                is ($AddressBuilder->_postalCode()->{'PostalCode'},'09876','call _postalCode with valid Value');
                is ($AddressBuilder->_city()->{'City'},'Neustadt','call _city with valid Value');
                is ($AddressBuilder->_countryCode()->{'CountryCode'},'DE','call _countryCode with valid Value');
                testStateProvinceCode($AddressBuilder);
                testUnDefinedTaxIdentificationNumber($AddressBuilder);
                # caused by the negative tests, the "AddressBuilder" object  must be reloaded
                $hShippingAddress = $DataCollectors->getAddress('Address');
                $AddressBuilder = DE_EPAGES::UPS::API::XML::Builder::AddressBuilder->new($hShippingAddress);
                testAssemble($AddressBuilder);
                # TaxID is only available in Shopadress
                my $hShopAddress = $DataCollectors->getAddress('ShopAddress');
                $AddressBuilder = DE_EPAGES::UPS::API::XML::Builder::AddressBuilder->new($hShopAddress);
                testDefinedTaxIdentificationNumber($AddressBuilder);
            };

            if ( ExistsError() ) {
                my $Error = GetError();
                fail( 'Error: ' . $Error->longMessage );
            }

            # shutDown
            ok( Cleanup(), 'cleanup' );
            done_testing();
        }
    );
}
sub Cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

sub Init {
    return 1 if ExistsObjectByPath($ROOTPATH) && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    LoadRootObject()->insertChild( 'Object', { Alias => 'TestStore' } );
    ImportObjects( 'importOrder.xml', $ROOTPATH );
    return 1;
}
sub testAssemble {
    my ($AddressBuilder) = @_;
    my $hAddress = $AddressBuilder->assemble();
    is($hAddress->{'Phone'}->{'Number'},'040123456789','assemble PhoneNumber');
    is($hAddress->{'AttentionName'},'Herr','assemble AttentionName');
    is($hAddress->{'Name'},'Max Mustermann','assemble Name');
    is($hAddress->{'Address'}->{'PostalCode'},'09876','assemble PostalCode');
    is($hAddress->{'Address'}->{'AddressLine'},'Blumenweg 2','assemble AddressLine');
    is($hAddress->{'Address'}->{'StateProvinceCode'},'Deutschland','assemble StateProvinceCode');
    is($hAddress->{'Address'}->{'City'},'Neustadt','assemble Neutstadt');
    is($hAddress->{'Address'}->{'CountryCode'},'DE','assemble CountryCode');
}
sub testPhone {
    my ($AddressBuilder) = @_;
    is ($AddressBuilder->_phone()->{'Phone'}->{'Number'},'040123456789','call _phone with valid Value');
    $AddressBuilder->{'Address'}->{'Phone'} = undef;
    is (%{$AddressBuilder->_phone()},0,'call _phone with unvalid Value');
}

sub testAttentionName {
    my ($AddressBuilder) = @_;
    is ($AddressBuilder->_attentionName()->{'AttentionName'},'Herr','call _attentionName with valid Value');
    $AddressBuilder->{'Address'}->{'Salutation'} = undef;
    is (%{$AddressBuilder->_attentionName()},0,'call _attentionName with unvalid Value');
}

sub testStateProvinceCode {
    my ($AddressBuilder) = @_;
    is ($AddressBuilder->_stateProvinceCode()->{'StateProvinceCode'},'Deutschland','call _stateProvinceCode with valid Value');
    $AddressBuilder->{'Address'}->{'State'} = undef;
    is (%{$AddressBuilder->_stateProvinceCode()},0,'call _stateProvinceCode with unvalid Value');
}
sub testDefinedTaxIdentificationNumber {
    my ($AddressBuilder) = @_;
    is ($AddressBuilder->_taxIdentificationNumber()->{'TaxIdentificationNumber'},'123456789','call _taxIdentificationNumber with valid Value');
}
sub testUnDefinedTaxIdentificationNumber {
    my ($AddressBuilder) = @_;
    is (%{$AddressBuilder->_taxIdentificationNumber()},0,'call _taxIdentificationNumber with unvalid Value');
}

RunScript( Sub => \&Main );

1;