use strict;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw (
    ExistsObjectByPath
    LoadObjectByPath
    LoadRootObject
);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );
use DE_EPAGES::UPS::API::DataCollectors;
use DE_EPAGES::UPS::API::XML::Builder::PackageBuilder;
use Readonly;
use Test::More;
use Test::Exception;

use Data::Dumper;
Readonly my $ROOTPATH => '/TestStore';

## no critic (ProhibitLongLines)
sub Main {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {

            # startUp
            ok( Init(), 'initialization' );

            # tests
            eval {
                my $ShippingLabelUPS = LoadObjectByPath($ROOTPATH.'/TestShop/Customers/1001/Orders/1002/ShippingLabels/1001'); ## no critic (ProhibitPathConcatenation)
                isa_ok($ShippingLabelUPS, 'DE_EPAGES::UPS::API::Object::ShippingLabelUPS');
                my $DataCollectors = DE_EPAGES::UPS::API::DataCollectors->new($ShippingLabelUPS);
                isa_ok($DataCollectors, 'DE_EPAGES::UPS::API::DataCollectors');
                my $hShippingLabel = $DataCollectors->getShippingLabelData();
                my $PackageBuilder = DE_EPAGES::UPS::API::XML::Builder::PackageBuilder->new($hShippingLabel);
                isa_ok($PackageBuilder, 'DE_EPAGES::UPS::API::XML::Builder::PackageBuilder');
                testDescription($PackageBuilder);
                testWeight($PackageBuilder);
                testPackaging($PackageBuilder);
                TestUnitOfWeight($PackageBuilder);
                testPackageWeight($PackageBuilder);
                TestUnitOfLength($PackageBuilder);
                testSortedDimensions($PackageBuilder);
                testDimensions($PackageBuilder);
                # assemble Test
                $hShippingLabel = $DataCollectors->getShippingLabelData();
                $PackageBuilder = DE_EPAGES::UPS::API::XML::Builder::PackageBuilder->new($hShippingLabel);
                testAssemble($PackageBuilder);
            };

            if ( ExistsError() ) {
                my $Error = GetError();
                fail( 'Error: ' . $Error->longMessage );
            }

            # shutDown
            ok( Cleanup(), 'cleanup' );
            done_testing();
        }
    );
}

sub Cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

sub Init {
    return 1 if ExistsObjectByPath($ROOTPATH) && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    LoadRootObject()->insertChild( 'Object', { Alias => 'TestStore' } );
    ImportObjects( 'importOrder.xml', $ROOTPATH );
    return 1;
}
## no critic (ProhibitPrivateSubroutineUse)
sub testDescription {
    my ($PackageBuilder) = @_;
    is ($PackageBuilder->_description()->{'Description'},'ShippingLabelUPS Description','call _description() with valid Data');
    $PackageBuilder->{'ShippingLabel'}->{'PackageDescription'} = undef;
    is ($PackageBuilder->_description()->{'Description'},undef,'call _description() with unvalid Data');
}
sub testPackaging {
    my ($PackageBuilder) = @_;
    is ($PackageBuilder->_packaging()->{'Packaging'}->{'Code'},'02','call _packaging()->{Packaging}->{Code}');
}
sub testWeight {
    my ($PackageBuilder) = @_;
    is ($PackageBuilder->_weight()->{'Weight'},'1.1234','call _weight() with valid Data');
    $PackageBuilder->{'ShippingLabel'}->{'TotalWeight'} = undef;
    is ($PackageBuilder->_weight()->{'Weight'},undef,'call _weight() with unvalid Data');
}
sub TestUnitOfWeight {
    my ($PackageBuilder) = @_;
    is ($PackageBuilder->_unitOfWeight()->{'UnitOfMeasurement'}->{'Code'},'KGS','call _unitOfWeight()->{UnitOfMeasurement}->{Code}');
}
sub TestUnitOfLength {
    my ($PackageBuilder) = @_;
    is ($PackageBuilder->_unitOfLength()->{'UnitOfMeasurement'}->{'Code'},'CM','call _unitOfLength()->{Packaging}->{Code}');
}
sub testPackageWeight {
    my ($PackageBuilder) = @_;
    $PackageBuilder->{'ShippingLabel'}->{'TotalWeight'} = '9.8765';
    is ($PackageBuilder->_packageWeight()->{'PackageWeight'}->{'UnitOfMeasurement'}->{'Code'},'KGS','call _packageWeight()->{UnitOfMeasurement}->{Code}');
    is ($PackageBuilder->_packageWeight()->{'PackageWeight'}->{'Weight'},'9.8765','call _packageWeight()->{Weight}');
}
sub testSortedDimensions {
    my ($PackageBuilder) = @_;
    my @sortedDimensions = $PackageBuilder->_sortedDimensions();
    is ($sortedDimensions[0],'3333','call _sortedDimensions() biggest Value');
    is ($sortedDimensions[1],'2222','call _sortedDimensions() middel Value');
    is ($sortedDimensions[2],'1111','call _sortedDimensions() smalest Value');
}

sub testDimensions {
    my ($PackageBuilder) = @_;
    ## no critic (RequireInterpolationOfMetachars)
    is ($PackageBuilder->_dimensions()->{'Dimensions'}->{'UnitOfMeasurement'}->{'Code'},'CM',' call $PackageBuilder->_dimensions()->{Dimensions}->{UnitOfMeasurement}->{Code}');
    is ($PackageBuilder->_dimensions()->{'Dimensions'}->{'Length'},'3333',' call $PackageBuilder->_dimensions()->{Dimensions}->{Length}');
    is ($PackageBuilder->_dimensions()->{'Dimensions'}->{'Width'},'2222',' call $PackageBuilder->_dimensions()->{Dimensions}->{Width}');
    is ($PackageBuilder->_dimensions()->{'Dimensions'}->{'Height'},'1111',' call $PackageBuilder->_dimensions()->{Dimensions}->{Height}');
}

sub testAssemble {
    my ($PackageBuilder) = @_;
    my $assemble = $PackageBuilder->assemble();
    is (scalar keys %{$assemble->{'Package'}},4,'number of subitems in Package');
    is (scalar keys %{$assemble->{'Package'}->{'Dimensions'}},4,'number of subitems in Package/Dimensions');
    is (scalar keys %{$assemble->{'Package'}->{'Dimensions'}},4,'number of subitems in Package/Dimensions');
    is (scalar keys %{$assemble->{'Package'}->{'Packaging'}},1,'number of subitems in Package/Dimensions');
    is (scalar keys %{$assemble->{'Package'}->{'PackageWeight'}},2,'number of subitems in Package/Dimensions');
}
RunScript( Sub => \&Main );

1;