use strict;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw (
    ExistsObjectByPath
    LoadObjectByPath
    LoadRootObject
);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );
use DE_EPAGES::UPS::API::DataCollectors;
use DE_EPAGES::UPS::API::XML::Builder::ShipmentBuilder;
use Readonly;
use Test::More;
use Test::Exception;

Readonly my $ROOTPATH => '/TestStore';
## no critic (ProhibitLongLines)
sub Main {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {

            # startUp
            ok( Init(), 'initialization' );

            # tests
            eval {
                my $ShippingLabelUPS = LoadObjectByPath($ROOTPATH.'/TestShop/Customers/1001/Orders/1002/ShippingLabels/1001'); ## no critic (ProhibitPathConcatenation)
                isa_ok($ShippingLabelUPS, 'DE_EPAGES::UPS::API::Object::ShippingLabelUPS');
                my $DataCollectors = DE_EPAGES::UPS::API::DataCollectors->new($ShippingLabelUPS);
                isa_ok($ShippingLabelUPS->get('Order'), 'DE_EPAGES::Order::API::Object::CustomerOrder');
                isa_ok($DataCollectors, 'DE_EPAGES::UPS::API::DataCollectors');
                my $ShippingLabelData = $DataCollectors->getShippingLabelData();
                my $ShipmentServiceOptionsBuilder = DE_EPAGES::UPS::API::XML::Builder::ShipmentServiceOptionsBuilder->new($ShippingLabelData);
                Test_saturdayDeliveryIndicator($ShipmentServiceOptionsBuilder);
                Test_carbonNeutral($ShipmentServiceOptionsBuilder);
                Test_confirmation($ShipmentServiceOptionsBuilder);
                Test_assemble($ShipmentServiceOptionsBuilder);
            };

            if ( ExistsError() ) {
                my $Error = GetError();
                fail( 'Error: ' . $Error->longMessage );
            }

            # shutDown
            ok( Cleanup(), 'cleanup' );
            done_testing();
        }
    );
}

sub Cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

sub Init {
    return 1 if ExistsObjectByPath($ROOTPATH) && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    LoadRootObject()->insertChild( 'Object', { Alias => 'TestStore' } );
    ImportObjects( 'importOrder.xml', $ROOTPATH );
    return 1;
}

## no critic (ProhibitPrivateSubroutineUse)
sub Test_saturdayDeliveryIndicator {
    my ($ShipmentServiceOptionsBuilder) = @_;
    my $TestVar = $ShipmentServiceOptionsBuilder->_saturdayDeliveryIndicator();
    is($TestVar->{'SaturdayDeliveryIndicator'},'0','saturdayDeliveryIndicator positiv');
    $ShipmentServiceOptionsBuilder->{'ShippingLable'}{'SaturdayDelivery'} = 0;
    $TestVar = $ShipmentServiceOptionsBuilder->_saturdayDeliveryIndicator();
    is($TestVar->{'SaturdayDeliveryIndicator'},undef,'saturdayDeliveryIndicator negativ');
}
sub Test_carbonNeutral {
    my ($ShipmentServiceOptionsBuilder) = @_;
    my $TestVar = $ShipmentServiceOptionsBuilder->_carbonNeutral();
    is($TestVar->{'UPSCarbonNeutral'},'0','UPSCarbonNeutral positiv');
    $ShipmentServiceOptionsBuilder->{'ShippingLable'}{'UPSCarbonNeutral'} = 0;
    $TestVar = $ShipmentServiceOptionsBuilder->_carbonNeutral();
    is($TestVar->{'UPSCarbonNeutral'},undef,'UPSCarbonNeutral negativ');
}

sub Test_confirmation {
    my ($ShipmentServiceOptionsBuilder) = @_;
    my $TestVar = $ShipmentServiceOptionsBuilder->_confirmation();
    is($TestVar->{'DeliveryConfirmation'}{'DCISType'},'3','AdultSignatureRequired');

    $ShipmentServiceOptionsBuilder->{'ShippingLable'}{'AdultConfirmation'} = 0;
    $TestVar = $ShipmentServiceOptionsBuilder->_confirmation();
    is($TestVar->{'DeliveryConfirmation'}{'DCISType'},'2','SignatureRequired');
    $ShipmentServiceOptionsBuilder->{'ShippingLable'}{'DeliveryConfirmation'} = 0;
    $TestVar = $ShipmentServiceOptionsBuilder->_confirmation();
    is($TestVar->{'DeliveryConfirmation'},undef,'_confirmation negativ (with negativ AdultConfirmation');

    $ShipmentServiceOptionsBuilder->{'ShippingLable'}{'AdultConfirmation'} = 1;
    $TestVar = $ShipmentServiceOptionsBuilder->_confirmation();
    is($TestVar->{'DeliveryConfirmation'},undef,'_confirmation negativ (with positiv AdultConfirmation');
}

sub Test_assemble {
    my ($ShipmentServiceOptionsBuilder) = @_;
    # all negativ
    $ShipmentServiceOptionsBuilder->{'ShippingLable'}{'SaturdayDelivery'} = 0;
    $ShipmentServiceOptionsBuilder->{'ShippingLable'}{'UPSCarbonNeutral'} = 0;
    $ShipmentServiceOptionsBuilder->{'ShippingLable'}{'DeliveryConfirmation'} = 0;
    $ShipmentServiceOptionsBuilder->{'ShippingLable'}{'AdultSignatureRequired'} = 0;
    my $TestVar = $ShipmentServiceOptionsBuilder->assemble();
    is(%{$TestVar},0,'assemble negativ');
    $ShipmentServiceOptionsBuilder->{'ShippingLable'}{'SaturdayDelivery'} = 1;
    $TestVar = $ShipmentServiceOptionsBuilder->assemble();
    is($TestVar->{'ShipmentServiceOptions'}{'SaturdayDeliveryIndicator'},0,'{ShipmentServiceOptions}{SaturdayDeliveryIndicator positiv');
    is($TestVar->{'ShipmentServiceOptions'}{'DeliveryConfirmation'},undef,'{ShipmentServiceOptions}{DeliveryConfirmation} negativ');
    is($TestVar->{'ShipmentServiceOptions'}{'UPSCarbonNeutral'},undef,'{ShipmentServiceOptions}{UPSCarbonNeutral} negativ');
    $ShipmentServiceOptionsBuilder->{'ShippingLable'}{'SaturdayDelivery'} = 0;
    $ShipmentServiceOptionsBuilder->{'ShippingLable'}{'UPSCarbonNeutral'} = 1;
    $TestVar = $ShipmentServiceOptionsBuilder->assemble();
    is($TestVar->{'ShipmentServiceOptions'}{'SaturdayDeliveryIndicator'},undef,'{ShipmentServiceOptions}{SaturdayDeliveryIndicator negativ');
    is($TestVar->{'ShipmentServiceOptions'}{'DeliveryConfirmation'},undef,'{ShipmentServiceOptions}{DeliveryConfirmation} negativ');
    is($TestVar->{'ShipmentServiceOptions'}{'UPSCarbonNeutral'},0,'{ShipmentServiceOptions}{UPSCarbonNeutral} postiv');
}
RunScript( Sub => \&Main );

1;